# -*- coding: UTF-8 -*-
#
# Copyright (c) 2009 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Authors: Hervé Cauwelier <herve@itaapy.com>
#
# This file is part of Lpod (see: http://lpod-project.org).
# Lpod is free software; you can redistribute it and/or modify it under
# the terms of either:
#
# a) the GNU General Public License as published by the Free Software
#    Foundation, either version 3 of the License, or (at your option)
#    any later version.
#    Lpod is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License
#    along with Lpod.  If not, see <http://www.gnu.org/licenses/>.
#
# b) the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#    http://www.apache.org/licenses/LICENSE-2.0
#

# Import from lpod
from element import FIRST_CHILD
from element import register_element_class, odf_create_element, odf_element
from paragraph import odf_create_paragraph
from utils import Boolean



def odf_create_toc(title=u"Table of Contents", name=None, protected=True,
        outline_level=None, style=None):
    """Create a table of contents. Default parameters are what most people
    use: Protected from manual modifications and not limited in title levels.

    The name is mandatory and derived automatically from the title if not
    given. Provide one if you insert several TOCs in the same document.

    Arguments:

        title -- unicode

        name -- unicode

        protected -- bool

        outline_level -- int

        style -- str

    Return: odf_toc
    """
    element = odf_create_element('<text:table-of-content/>')
    # XXX
    if name is None:
        name = u"%s1" % title
        element.set_attribute('text:name', name)
    element.set_attribute('text:protected', Boolean.encode(protected))
    if style:
        element.set_attribute('text:style-name', style)
    if outline_level:
        element.set_toc_outline_level(outline_level)
    if title:
        element.append_element(odf_create_index_title(title))
    return element



def odf_create_index_body():
    return odf_create_element('<text:index-body/>')



def odf_create_index_title(title=None):
    """Create an index title

    Arguments:

        title -- unicode

    Return: odf_element
    """
    element = odf_create_element('<text:index-title/>')
    if title:
        title = odf_create_paragraph(text=title)
        element.append_element(title)
    return element



class odf_toc(odf_element):

    def get_formatted_text(self, context):
        index_body = self.get_element('text:index-body')

        if index_body is None:
            return u''

        if context["rst_mode"]:
            return u"\n.. contents::\n"

        result = []
        for element in index_body.get_children():
            if element.get_tagname() == 'text:index-title':
                for element in element.get_children():
                    result.append(element.get_formatted_text(context))
                result.append(u'\n')
            else:
                result.append(element.get_formatted_text(context))
        result.append('\n')
        return u''.join(result)


    def get_toc_title(self):
        index_title = self.get_element('text:index-title')
        if index_title is None:
            return
        return index_title.get_text_content()


    def set_toc_title(self, title):
        index_title = self.get_element('text:index-title')
        if index_title is None:
            index_title = odf_create_index_title(title)
            self.append_element(index_title)
        else:
            index_title.set_text_content(title)


    def get_toc_outline_level(self):
        source = self.get_element('text:table-of-content-source')
        if source is None:
            return None
        return source.get_attribute('text:outline-level')


    def set_toc_outline_level(self, level):
        source = self.get_element('text:table-of-content-source')
        if source is None:
            source = odf_create_element('<text:table-of-content-source/>')
            self.insert_element(source, FIRST_CHILD)
        source.set_attribute('text:outline-level', str(int(level)))


    def toc_fill(self, document=None):
        """Fill the TOC with the titles found in the document. A TOC is not
        contextual so it will catch all titles before and after its insertion.

        If the TOC is not attached to a document, attach it beforehand or
        provide one as argument.

        Arguments:

        document -- odf_document
        """
        # Find the body
        if document is not None:
            content = document.get_content()
            body = content.get_body()
        else:
            body = self.get_body()
        if body is None:
            raise ValueError, "the TOC must be related to a document somehow"

        # Clean the old index-body
        index_body = self.get_element('text:index-body')
        if index_body:
            self.delete_element(index_body)
        index_body = odf_create_index_body()
        self.append_element(index_body)

        # Auto fill the index
        # 1. The title: "Table Of Contents"
        title = self.get_attribute('text:name')
        if not title:
            title = u"Table of Contents"
        title = odf_create_index_title(title)
        index_body.append_element(title)
        #2. The section
        level_indexes = {}
        for heading in body.get_heading_list():
            level = heading.get_outline_level()
            number = []
            # 1. l < level
            for l in range(1, level):
                index = level_indexes.setdefault(l, 1)
                number.append(unicode(index))
            # 2. l == level
            index = level_indexes.setdefault(level, 0) + 1
            level_indexes[level] = index
            number.append(unicode(index))
            # 3. l > level
            for l in range(level + 1, 11):
                if level_indexes.has_key(l):
                    del level_indexes[l]
            number = u'.'.join(number)
            number += u'.'
            # Make the title
            title = u"%s %s" % (number, heading.get_text())
            paragraph = odf_create_paragraph(text=title)
            index_body.append_element(paragraph)



register_element_class('text:table-of-content', odf_toc)
