# -*- coding: utf-8 -*-
#
# Generic Shell
# Copyright (C) 2010 Sébastien Dumont
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Import from the Standard Library
from sys import exit, stdout


##############################################################################
# Basic command
##############################################################################
class ShellCommand(object):
    """basic command
    """
    MIN_ARG = 0
    MAX_ARG = 0

    def process(self, args):
        self.validate_arguments(args)


    def validate_arguments(self, args):
        if len(args) < self.MIN_ARG:
            raise TooFewArgError
        elif len(args) > self.MAX_ARG:
            raise TooManyArgError


    def help(self):
        return self.__doc__.strip()



class HelpCommand(ShellCommand):
    """ [command]: list commands
    """
    MIN_ARG = 0
    MAX_ARG = 1

    def __init__(self, shell):
        ShellCommand.__init__(self)
        self.command_list = shell.command_list


    def process(self, args):
        ShellCommand.process(self, args)
        for command_name, command in self.command_list.iteritems():
            print("%s %s" % (command_name, command.help()))


    def validate_arguments(self, args):
        ShellCommand.validate_arguments(self, args)
        if args and args[0] not in self.command_list:
            raise CommandNotFoundError



class ExitCommand(ShellCommand):
    """ [command]: exit program
    """
    MIN_ARG = 0
    MAX_ARG = 0

    def process(self, args):
        exit(0)



##############################################################################
# Basic shell
##############################################################################
class Shell(object):

    def __init__(self, command_list = {}, start_string = '>>'):
        # Set basic commands
        self.command_list = {}
        self.command_list['help'] = HelpCommand(self)
        self.command_list['quit'] = ExitCommand()
        self.command_list.update(command_list)
        self.start_string = start_string


    def shell_loop(self):
        while True:
            # get stdin
            stdout.write(self.start_string)
            try:
                line = raw_input()
            except EOFError:
                break
            line = line.split()
            if line:
                command, args = line[0], line[1:]
                try:
                    self.run_command(command, args)
                except CommandError, msg:
                    print msg


    def run_command(self, command, args):
        # TODO remove debug
        # print "DEBUG command: '%s' args: '%s'" % (command, args)
        if command in self.command_list:
            self.command_list[command].process(args)
        else:
            raise CommandNotFoundError



##############################################################################
# Basic Exceptions
##############################################################################
class CommandError(Exception):
    def __str__(self):
        return 'Command Error'



class CommandNotFoundError(CommandError):
    def __str__(self):
        return 'Command not found'



class TooFewArgError(CommandError):
    def __str__(self):
        return '%s: Too Few Arguments' % CommandError.__str__(self)



class TooManyArgError(CommandError):
    def __str__(self):
        return '%s: Too Many Arguments' % CommandError.__str__(self)



